@@ -29,26 +29,25 @@ void run_rgb_timing()
   */
  rgb_timing_test = create_test(&rgb_timing_dtest,tsamples,psamples,&rgb_timing);
 
- /*
-  * Set any GLOBAL data used by the test.
-  */
-
- /*
-  * Show the standard test header for this test.
-  */
- show_test_header(&rgb_timing_dtest,rgb_timing_test);
-
  /*
   * Call the actual test that fills in the results struct.
   */
  rgb_timing(rgb_timing_test,&timing);
 
  /*
-  * Display the results.
+  * Save this for display in 
   */
+ rng_avg_time_nsec = timing.avg_time_nsec;
+ rng_rands_per_second = timing.rands_per_sec;
+
+ /*
+  * Display the results.
  printf("#========================================================================\n");
  printf("# rgb_timing() test using the %s generator \n",gsl_rng_name(rng));
  printf("# Average time per rand = %e nsec.\n",timing.avg_time_nsec);
  printf("# Rands per second = %e.\n",timing.rands_per_sec);
+  */
+
+ destroy_test(&rgb_timing_dtest,rgb_timing_test);
 
 }
